/*******************************************************************/
/*

Demo-Software for smart cameras from Vision Components
------------------------------------------------------


Program Description:
--------------------

This program is for color cameras that do not have a hardware color input LUT
(VC4016, VC4018, SBC4012 for example).
It calculates the white balance correction values and programs the software
LUT and saves an image after applying the LUT.

Process :
- calculate the corrected red, green and blue values with WhiteBalanceValues()
- use these values to initialise a color table with init_color_table()
- take picture
- apply the values to the bayer pattern with clut_bayer()
- save RGB image to flash

NOTE :
The upleft pixel of the sensor of the VC4016 and SBC4016 cameras is a green pixel,
unlike on other color cameras where the upleft pixel is red. VC color algorithms
need an upleft red pixel, this is why on VC4016 and SBC4016 cameras, an offset
of one line has to be added to the start address of the bayer pattern.


Revision History:
-----------------

   Date           Version    Author  Changes
   Nov 14, 2007   1.00       LB      Initial Version
   Dec 03, 2008   2.00       LB      Bug corrections, compatible with VC4016 / 4018 / 4012
   Jan 13, 2009   2.01       LB      Note for VC4016 / SBC4016

*/
/*******************************************************************/
#define NEW_IMAGE_VAR			/* important for color image variables	*/

#include <sysvar.h>
#include <vcrt.h>
#include <vclib.h>
#include <flib.h>
#include <macros.h>

/*******************************************************************/


void main(void)
{
	image bayer_wb, bayer1, bayer2, rgb;
	I32 red, green, blue, bright, dx, dy, res;
	U8 *BayerStart;
	U8 *table;

	/*******************************************************************************/
	/*                 PLEASE INITIALIZE YOUR LICENCE CODES                        */
	/*                    To activate library functions                            */
	/*                                                                             */
	/* VCLIB :         init_licence("T1122334455") or init_licence("LT1122334455") */
	/* Extension Lib : init_licence("E1122334455") or init_licence("LE1122334455") */
	/* Color Lib :     init_licence("C1122334455") or init_licence("LC1122334455") */
	/* etc.                                                                        */
	/*                                                                             */
	/*      init_licence() returns 0 if licence code is OK, -5 otherwise           */
	/*                                                                             */
	/* Example :                                                                   */
	/*                                                                             */
	/**/ 	print("Init VCLIB = %d\n", init_licence("T1122334455"));			 /**/
	/**/ 	print("Init Color Lib = %d\n", init_licence("C1122334455"));         /**/
	/*******************************************************************************/

	// Size of the image
	dx = 640;
	dy = 480;

	// If VC4016 or SBC4016, first red pixel one line below
	if (getvar(MODEL) == 4016)
		BayerStart = (U8*)(ScrGetCaptPage + ScrGetPitch);
	else
		BayerStart = (U8*)(ScrGetCaptPage);

	// Allocate LUT
	table = vcmalloc(1024);

	// Define Image variables
	ImageAssignC(&bayer1, BayerStart, IMAGE_BAYER, dx, dy, ScrGetPitch, (U8 *)0, (U8 *)0);
	ImageAssignC(&bayer_wb, BayerStart, IMAGE_BAYER, 256, 256, ScrGetPitch, (U8 *)0, (U8 *)0);

	// Allocate RGB color image and 2nd bayer image
	ImageAllocate(&rgb, 	IMAGE_RGB, dx, dy);
	ImageAllocate(&bayer2, 	IMAGE_BAYER, dx, dy);

	// Set RGB image to medium grey
	cset(&rgb, 0, 0, 128);

	// Put a white/grey background
	print("\nPut a white or grey background and press a key...\n\n"); getchar();

	// Calculation of white balance values
	// Maximum brightness of image, should not exceed 180
	print("Adjust brightness between 120 and 180 and press a key\n");

	while(!kbhit())
	{
		tpict();
		bright = WhiteBalanceValues (&bayer_wb, &red, &green, &blue);
		print("Brightness :  %3d\n", bright);
	}

	rs232rcv();
	print("\n");

	// Calculated values
	print("Calculated values\n");
	print("Red: %d\nGreen: %d\nBlue: %d\n\n", red, green, blue);

	// Initialisation of color input LUT
	res = init_color_table ((U32)red, (U32)green, (U32)blue, table);
	print("init : %d\nred=%d, green=%d, blue=%d\n", res, red, green, blue);

	// Switch to live mode
	print("Press a key to go to live mode...\n");
	rs232rcv();
	vmode(vmLive);

	print("Press a key to take picture...\n");
	rs232rcv();

	// Take picture
	tpict();

	// Apply LUT to image
	clut_bayer(&bayer1, &bayer2, table);

	// Create RGB image
	BayerToRGB(&bayer2, &rgb);

	// Save corrected RGB image on flash
	print("Saving RGB image...\n");
	fwrite_image("fd:/rgb.bmp", &rgb);

	// Free memory
	ImageFree(&bayer2);
	ImageFree(&rgb);
	vcfree(table);
 
}
