/*******************************************************************/
/*

Demo-Software for smart cameras from Vision Components
------------------------------------------------------

Program: Klaus Schneider,  VC


Program Description:
--------------------

The file corr.c is an example program for the small kernel correlation 
routine vc_corr1().

Maximum kernel size:

        16x16x8 bit
        10x25x8 bit
        or:
        kx * ky <= 256

The program takes a 64 x 64 sample (Sample1) and reduce it to 
the size 16 x 16 (Sample2) by using two times the function pyramid().    

The same is done for the search area (Search1). The result is stored 
in the image variable Search2. The search area is defined by SearchX, 
SearchY, SearchDX and SearchDY. 

Both image variables (Sample2 and Search2) are stored in a new image 
page called Work. 

The image variable Pattern is the same as Sample2 but with the 
correct pattern size (16 x 16). Search is the same as Search2 
but also with the correct size (Search1.dx/4 and Search1.dy/4).

After taking an image the program reduce the search area and do 
the correlation with the image variable Search and Pattern.

The result of the programm is the pattern position (PosX, PosY) and
the correlation result (CorrRes).

*/
/*******************************************************************/

#include <vcrt.h>
#include <vclib.h>
#include <macros.h>
#include <sysvar.h>
#include <flib.h>

/*******************************************************************/

void main(void)

  {
  int   Work, PosX, PosY, CorrRes, min_contr=5, min_corr=0, posy=0; 
  int   SearchX, SearchY, SearchDX, SearchDY;
  image Search1, Search2, SearchOvl, Sample1, Sample2, SampleOvl;
  image Screen, Overlay, Pattern, Search, Result;

    /*******************************************************************************/
    /*                 PLEASE INITIALIZE YOUR LICENCE CODES                        */
    /*                    To activate library functions                            */
    /*                                                                             */
    /* VCLIB :         init_licence("T1122334455") or init_licence("LT1122334455") */
    /* Extension Lib : init_licence("E1122334455") or init_licence("LE1122334455") */
    /* Color Lib :     init_licence("C1122334455") or init_licence("LC1122334455") */
    /* etc.                                                                        */
    /*                                                                             */
    /*      init_licence() returns 0 if licence code is OK, -5 otherwise           */
    /*                                                                             */
    /* Example :                                                                   */
    /*                                                                             */
	/**/ 	print("Init VCLIB = %d\n", init_licence("T1122334455"));			 /**/
    /*******************************************************************************/

  /* full processor power */
  set_overlay_bit(0, 255, 0, 2);
  set_ovlmask(255);
  vmode(vmOvlLive);  

  /* define serach area */
  SearchDX = 256; 
  SearchDY = 256; 
  SearchX  = (ScrGetColumns-SearchDX)/2;
  SearchY  = (ScrGetRows   -SearchDY)/2;

  /* Set logical page */
  ScrSetLogPage((int)ScrGetPhysPage);
  OvlSetLogPage((int)OvlGetPhysPage);

  /* clear overlay */
  OvlClearAll;

  /* define overlay rectangle for matching results */
  ImageAssign(&Result, OvlBitAddr(SearchX+SearchDX+30,SearchY), 30, SearchDY, OvlGetPitch);  
  
  /* define overlay search area */
  ImageAssign(&SearchOvl, OvlBitAddr(SearchX,SearchY), SearchDX, SearchDY, OvlGetPitch);  
  
  /* define search area */
  ImageAssign(&Search1, ScrByteAddr(SearchX,SearchY), SearchDX, SearchDY, ScrGetPitch);  
  
  /* define overlay sample area */
  ImageAssign(&SampleOvl, OvlBitAddr(OvlGetColumns/2-32,OvlGetRows/2-32), 64, 64, OvlGetPitch);  
  
  /* define sample area */
  ImageAssign(&Sample1, ScrByteAddr(ScrGetColumns/2-32,ScrGetRows/2-32), 64, 64, ScrGetPitch);  
  
  /* allocate working page */
  Work = DRAMScreenMalloc();

  /* set logical addr. */
  ScrSetLogPage(Work);

  /* define image variable at the working page */
  ImageAssign(&Sample2, ScrByteAddr(0,  0), 64, 64, ScrGetPitch);  
  ImageAssign(&Pattern, ScrByteAddr(0,  0), 16, 16, ScrGetPitch);  

  ImageAssign(&Search2, ScrByteAddr(0, 64), SearchDX, SearchDY, ScrGetPitch);  
  ImageAssign(&Search , ScrByteAddr(0, 64), SearchDX/4, SearchDY/4, ScrGetPitch);  


  /****************/
  /* store sample */
  /****************/

  /* Drawings */
  dframez(&SearchOvl);
  framez(&SampleOvl);
  markerz(&SampleOvl);

  pstr("place sample in center frame\n");
  pstr("press any key when ready\n");

  /* read empty buffer    */
  while(rbempty() != -1) rs232rcv();

  /* wait until user is ready */
  rs232rcv();

  /* take sample image */
  tpict();

  /* reduce sample size from 64 x 64 to 16 x 16 */
  pyramid(&Sample1,&Sample2);
  pyramid(&Sample2,&Sample2);
  
  pstr("sample stored\n");

  dframez(&Result);
  linez(&Result,2,posy,Result.dx-2,posy);
  
  /***************/
  /* correlation */
  /***************/

  do 
    {
    tpict();

    /* full processor power */
    vmode(vmOvlStill);

    /* reduce search area */
    pyramid(&Search1,&Search2);
    pyramid(&Search2,&Search2);

    CorrRes = vc_corr1(&Search, &Pattern, min_contr, min_corr, &PosX, &PosY);

    /* new drawings */
    if (CorrRes>0)
      {
      /* delete old drawings */
      framez(&SampleOvl);
      markerz(&SampleOvl);
      linez(&Result,2,posy,Result.dx-2,posy);

      /* calculate drawings */
      ImageAssign(&SampleOvl, OvlBitAddr(4*PosX+SearchX, 4*PosY+SearchY), 64, 64, OvlGetPitch);  
      posy=(1024-CorrRes)>>2;

      /* new drawings */
      framez(&SampleOvl);
      markerz(&SampleOvl);
      linez(&Result,2,posy,Result.dx-2,posy);
      }
    }
  while(rbempty() == -1);
  rs232rcv();

  DRAMPgFree(Work);
  }

/*******************************************************************/





