/************************************************************************************/
/*

Demo-Software for smart cameras from Vision Components
------------------------------------------------------

Program Description:
--------------------

Example of using the keypad with menus on the overlay.
The program draws menu items on the top of the screen, left and right
keys of the keypad allow to navigate into it.
ESC key of the keypad exits the program.

Revision History:
-----------------

   Date           Version    Author  Changes
   Mar 07, 2007   1.00       LB      Initial Version
   Jul 09, 2007   1.01       LB      Removed ReadKeyboard(), now integrated in VCLIB

*/
/************************************************************************************/

#include <vcrt.h>
#include <vclib.h>
#include <macros.h>
#include <sysvar.h>
#include <string.h>
#include <flib.h>

/************************************************************************************/

// Position of displayed image
#define DISPOFFSET_X 50
#define DISPOFFSET_Y 50
#define NUM_ITEMS     4

// Size of the characters, in pixels (muliplied by 8)
#define CHAR_SIZE 2

// Colors for overlay LUT
#define COLBLACK   0
#define COLWHITE  16
#define COLBLUE   24
#define COLRED    32
#define COLGREEN  40
#define COLGREY   48
#define COLCYAN   56
#define COLMAGE   64
#define COLDARK   72
#define COLYELLOW 80

#define COLBACKGROUND COLBLACK
#define COLHIGHLIGHT  COLRED

/************************************************************************************/

typedef struct      /* Characteristics of menu item */
{
  char *  ItemString;     // Text of menu item
  image ItemImage;        // Position and size of menu item, through an image variable

} MenuItem;

/************************************************************************************/

void     MoveDisplay      (void);
void     InitOverlay      (void);
void     ChangeColor      (image *area, I32 start, I32 end, I32 newcol);
void     HighlightItem    (MenuItem Item, int Color, int Highlight);
void     DispItems        (MenuItem * Item, int CharSize, int Color, int NumItems);
void     IniOverlayLUT    (void);
MenuItem CreateMenuItem   (char * TempString, int * Position, int CharSize);

/************************************************************************************/

void main(void)
  {
    int *NextPosition;
	int CurrentItem = 0;
	int FormerItem = 0;
	char response;
	MenuItem MainMenu[NUM_ITEMS];
    
	// Menu items strings
    char ItemString0[] = "Menu";
	char ItemString1[] = "Image";
	char ItemString2[] = "Test";
	char ItemString3[] = "Inspect";

    /*******************************************************************************/
    /*                 PLEASE INITIALIZE YOUR LICENCE CODES                        */
    /*                    To activate library functions                            */
    /*                                                                             */
    /* VCLIB :         init_licence("T1234567890") or init_licence("LT1234567890") */
    /* Extension Lib : init_licence("E1234567890") or init_licence("LE1234567890") */
    /* Color Lib :     init_licence("C1234567890") or init_licence("LC1234567890") */
    /* etc.                                                                        */
    /*                                                                             */
    /*      init_licence() returns 0 if licence code is OK, -5 otherwise           */
    /*                                                                             */
    /* Example :                                                                   */
    /*                                                                             */
	/**/ 	print("Init VCLIB = %d\n", init_licence("T1122334455"));			 /**/
    /*******************************************************************************/ 

	// Keypad
	INTERFACE_MODE(SERIAL);

    // Creating menu items, storing them all in a table
	*NextPosition = (8*CHAR_SIZE)/2;
    MainMenu[0] = CreateMenuItem(ItemString0, NextPosition, CHAR_SIZE);
	MainMenu[1] = CreateMenuItem(ItemString1, NextPosition, CHAR_SIZE);
	MainMenu[2] = CreateMenuItem(ItemString2, NextPosition, CHAR_SIZE);
	MainMenu[3] = CreateMenuItem(ItemString3, NextPosition, CHAR_SIZE);

    // Go to still mode 
    vmode(vmStill);
    tpwait();

    // Erase and initialize overlay
	InitOverlay();
	IniOverlayLUT();
 
    // Moving display
    MoveDisplay();
       
    /* Display Live and Overlay */
    vmode(vmOvlLive);

    // Displaying menu items
	DispItems(MainMenu, CHAR_SIZE, COLWHITE, NUM_ITEMS);

	// Highlight left item
	HighlightItem(MainMenu[0], COLHIGHLIGHT, 1);

	// Main loop, using the menu
	do
      {
        response = ReadKeyboard(1); 
    
        switch (response)
	      {
	        case KEY_LEFT  :
	           
                    FormerItem = CurrentItem;

			        if(CurrentItem > 0)
				      { CurrentItem-=1; }
				    else
				      { CurrentItem = NUM_ITEMS - 1; }

					// Highlight corresponding item
				    HighlightItem(MainMenu[CurrentItem], COLHIGHLIGHT, 1);

					// Remove highlighting from former item
					HighlightItem(MainMenu[FormerItem], COLBACKGROUND, 0);

					// Update overlay to avoid flickering
					display_update(UPDATE_OVLY);
     				wait_vsync();
	        
	                break;

	        case KEY_RIGHT :

			        FormerItem = CurrentItem;

			        if(CurrentItem < NUM_ITEMS - 1)
				      { CurrentItem+=1; }
				    else
				      { CurrentItem = 0; }

					// Highlight corresponding item
				    HighlightItem(MainMenu[CurrentItem], COLHIGHLIGHT, 1);

					// Remove highlighting from former item
					HighlightItem(MainMenu[FormerItem], COLBACKGROUND, 0);

					// Update overlay to avoid flickering
					display_update(UPDATE_OVLY);
     				wait_vsync();
	        
	                break;

	        case KEY_UP    : break;
	        case KEY_DOWN  : break;
	        case KEY_F1    : break;
	        case KEY_F2    : break;
	        case KEY_ESC   : goto END;

	        case KEY_ENTER : break;
	      } 

      } // do
    while(response != 'q');

    END:
	    InitOverlay();

  } 

/************************************************************************************/

void MoveDisplay(void)
  {
    image Display;

    ImageAssign(&Display, ScrGetCaptPage, ScrGetColumns, ScrGetRows, ScrGetPitch);

    /* Set capture page to new address */
    ScrSetCaptPage(ScrGetDispPage + DISPOFFSET_X + DISPOFFSET_Y * ScrGetPitch);
  
    /* Erase old capture page */
    set(&Display, 0); 

  }

/************************************************************************************/

void InitOverlay(void)
  {
    image Ovl;

	OvlSetLogPage((int)OvlGetPhysPage);

	ImageAssign(&Ovl, OvlByteAddr(0,0), DispGetColumns, DispGetRows, ScrGetPitch);
    set(&Ovl, 0);
	set_ovlmask(255);

  }

/************************************************************************************/

void ChangeColor (image *area, I32 start, I32 end, I32 newcol)
{
  I32 i, j, dx, dy;
  long addr, pitch;
  U8 *restrict ppix;

  
  addr=area->st - (8*CHAR_SIZE)/2;
  dx=area->dx + (8*CHAR_SIZE);
  dy=area->dy;
  pitch=(long)area->pitch;

  for (j=0; j<dy; j++)
  {
    ppix=(U8 *)(addr+(pitch*(long)j));

    for (i=0; i<dx; i++)
    {
      if ( (*ppix>=start) && (*ppix<=end) )
      {
        *ppix=newcol;
      }
      ppix++;
    }
  }
}

/************************************************************************************/

void HighlightItem(MenuItem Item, int Color, int Highlight)
  {
    if (Highlight) // Highlight item
      { ChangeColor (&Item.ItemImage, COLBACKGROUND, COLBACKGROUND, Color); }
	else  // Remove highlighting
	  { ChangeColor (&Item.ItemImage, COLHIGHLIGHT, COLHIGHLIGHT, Color); }

  }

/************************************************************************************/

void DispItems(MenuItem * Item, int CharSize, int Color, int NumItems)
  {
	int i;

    // Print strings to overlay
	for(i=0;i<NumItems;i++)
	  {
        chprint1(Item[i].ItemString, &Item[i].ItemImage, CharSize, CharSize, Color);
	  }

  }

/************************************************************************************/

void IniOverlayLUT(void)
{
  volatile I32 i;
  I32 color;
  volatile U32 table[256][3];
  volatile U32 *lut;


  // ini table
  for(i=0; i<256 ; i++)
  {	
    table[i][0]=0;	// RED		 
    table[i][1]=0;  // GREEN
    table[i][2]=0;	// BLUE		 
  }

  /******************************************************/
  /* Define Main Color (31 colors [8, 16, 24, ... 248]) */
  /******************************************************/

  color = COLBLACK;
  table[color][0] =  0;  // RED
  table[color][1] =  0;  // GREEN
  table[color][2] =  0;  // BLUE

  color = COLWHITE;
  table[color][0] = 240;  // RED
  table[color][1] = 240;  // GREEN
  table[color][2] = 240;  // BLUE
   
  color = COLBLUE;
  table[color][0] =  16;  // RED
  table[color][1] =   5;  // GREEN
  table[color][2] =  73;  // BLUE
   
  color = COLRED;
  table[color][0] = 250;  // RED
  table[color][1] =   0;  // GREEN
  table[color][2] =   0;  // BLUE
   
  color = COLGREEN;
  table[color][0] =  47;  // RED
  table[color][1] = 200;  // GREEN
  table[color][2] =  47;  // BLUE
   
  color = COLYELLOW;
  table[color][0] = 250;  // RED
  table[color][1] = 250;  // GREEN
  table[color][2] =   0;  // BLUE
   
  color = COLGREY;
  table[color][0] = 210;  // RED
  table[color][1] = 210;  // GREEN
  table[color][2] = 210;  // BLUE

  color = COLCYAN;
  table[color][0] =  47;  // RED
  table[color][1] = 191;  // GREEN
  table[color][2] = 191;  // BLUE

  color = COLMAGE;
  table[color][0] = 200;  // RED
  table[color][1] =  40;  // GREEN
  table[color][2] = 200;  // BLUE

  color = COLDARK;
  table[color][0] =  47;  // RED
  table[color][1] =  47;  // GREEN
  table[color][2] =  47;  // BLUE


  /******************************************************/
  /* programm overlay LUT                               */
  /******************************************************/

  lut = (volatile unsigned int *)VIRTX_OLUT;

  for(i=0; i<256 ; i++)
  {				 
    *lut++ = 
           ( (table[i][0])     ) +  /* RED   */
           ( (table[i][1]) << 8) +  /* GREEN */
           ( (table[i][2]) <<16) ;  /* BLUE  */
  }
}

/************************************************************************************/

MenuItem CreateMenuItem(char * TempString, int * Position, int CharSize)
  {
    image    TempImage;
    MenuItem TempItem;

	// Writing string to structure
	TempItem.ItemString = TempString;

	// Define image variable of the size of the printed string
    ImageAssign(&TempImage, OvlByteAddr(0,0) + *Position, 8*CharSize*strlen(TempString), 8*CharSize, ScrGetPitch);
	TempItem.ItemImage = TempImage;

	// Return value : next free position
    *Position = *Position + 8*CharSize*(strlen(TempString) + 1);

    // Return menu item
	return(TempItem);

  }

/************************************************************************************/

